@@ -1,30 +1,6 @@
-require 'constraints/group_url_constrainer'
-
-constraints(GroupUrlConstrainer.new) do
-  scope(path: ':id',
-        as: :group,
-        constraints: { id: Gitlab::Regex.namespace_route_regex },
-        controller: :groups) do
-    get '/', action: :show
-    patch '/', action: :update
-    put '/', action: :update
-    delete '/', action: :destroy
-  end
-end
-
 resources :groups, only: [:index, :new, :create]
 
-scope(path: 'groups/:id',
-      controller: :groups,
-      constraints: { id: Gitlab::Regex.namespace_route_regex }) do
-  get :edit, as: :edit_group
-  get :issues, as: :issues_group
-  get :merge_requests, as: :merge_requests_group
-  get :projects, as: :projects_group
-  get :activity, as: :activity_group
-end
-
-scope(path: 'groups/:group_id',
+scope(path: 'groups/*group_id',
       module: :groups,
       as: :group,
       constraints: { group_id: Gitlab::Regex.namespace_route_regex }) do
@@ -36,10 +12,20 @@ scope(path: 'groups/:group_id',
   resource :avatar, only: [:destroy]
   resources :milestones, constraints: { id: /[^\/]+/ }, only: [:index, :show, :update, :new, :create]
 
-  resources :labels, except: [:show], constraints: { id: /\d+/ } do
+  resources :labels, except: [:show] do
     post :toggle_subscription, on: :member
   end
 end
 
+scope(path: 'groups/*id',
+      controller: :groups,
+      constraints: { id: Gitlab::Regex.namespace_route_regex }) do
+  get :edit, as: :edit_group
+  get :issues, as: :issues_group
+  get :merge_requests, as: :merge_requests_group
+  get :projects, as: :projects_group
+  get :activity, as: :activity_group
+end
+
 # Must be last route in this file
-get 'groups/:id' => 'groups#show', as: :group_canonical, constraints: { id: Gitlab::Regex.namespace_route_regex }
+get 'groups/*id' => 'groups#show', as: :group_canonical, constraints: { id: Gitlab::Regex.namespace_route_regex }
